@@ -455,7 +455,8 @@ alm <- function(formula, data, subset, na.action,
         );
 
         # Get the scale value
-        scale <- scalerInternal(B, distribution, y, matrixXreg, mu, other);
+        # Suppress warnings for LASSO/RIDGE
+        scale <- suppressWarnings(scalerInternal(B, distribution, y, matrixXreg, mu, other));
 
         return(list(mu=mu,scale=scale,other=other,poly1=poly1));
     }
@@ -682,31 +683,20 @@ alm <- function(formula, data, subset, na.action,
                 CFValue <- meanFast(sqrt(abs(y-yFitted)));
             }
             else if(loss=="LASSO"){
-                B[] <- B / denominator;
-
-                if(interceptIsNeeded){
-                    CFValue <- (1-lambda) * sqrt(meanFast((y-yFitted)^2))/yDenominator + lambda * sum(abs(B[-1]))
-                }
-                else{
-                    CFValue <- (1-lambda) * sqrt(meanFast((y-yFitted)^2))/yDenominator + lambda * sum(abs(B))
-                }
-                # This is a hack. If lambda=1, then we only need the mean of the data
-                if(lambda==1){
-                    CFValue <- sqrt(meanFast((y-yFitted)^2));
-                }
+                CFValue <- (1-lambda) * meanFast((y-yFitted)^2) + lambda * sum(abs(B));
             }
             else if(loss=="RIDGE"){
-                B[] <- B / denominator;
+                B[] <- B * denominator;
 
                 if(interceptIsNeeded){
-                    CFValue <- (1-lambda) * sqrt(meanFast((y-yFitted)^2))/yDenominator + lambda * sqrt(sum(B[-1]^2))
+                    CFValue <- (1-lambda) * meanFast((y-yFitted)^2) + lambda * sqrt(sum(B[-1]^2))
                 }
                 else{
-                    CFValue <- (1-lambda) * sqrt(meanFast((y-yFitted)^2))/yDenominator + lambda * sqrt(sum(B^2))
+                    CFValue <- (1-lambda) * meanFast((y-yFitted)^2) + lambda * sqrt(sum(B^2))
                 }
                 # This is a hack. If lambda=1, then we only need the mean of the data
                 if(lambda==1){
-                    CFValue <- sqrt(meanFast((y-yFitted)^2));
+                    CFValue <- meanFast((y-yFitted)^2);
                 }
             }
             else if(loss=="custom"){
@@ -824,10 +814,8 @@ alm <- function(formula, data, subset, na.action,
             alpha <- nu <- size <- sigma <- shape <- lambdaBC <- nu <- 0.5;
             aParameterProvided <- TRUE;
     }
-    # LASSO / RIDGE loss
+    # LASSO loss
     if(any(loss==c("LASSO","RIDGE"))){
-        warning(paste0("Please, keep in mind that loss='",loss,
-                       "' is an experimental option. It might not work correctly."), call.=FALSE);
         if(is.null(ellipsis$lambda)){
             warning("You have not provided lambda parameter. We will set it to zero.", call.=FALSE);
             lambda <- 0;
@@ -860,12 +848,7 @@ alm <- function(formula, data, subset, na.action,
         xtol_rel <- ellipsis$xtol_rel;
     }
     if(is.null(ellipsis$algorithm)){
-        # if(recursiveModel){
-        # algorithm <- "NLOPT_LN_BOBYQA";
-        # }
-        # else{
         algorithm <- "NLOPT_LN_SBPLX";
-        # }
     }
     else{
         algorithm <- ellipsis$algorithm;
@@ -1317,6 +1300,15 @@ alm <- function(formula, data, subset, na.action,
     # The number of exogenous variables (no ARI elements)
     nVariablesExo <- nVariables;
 
+    if(any(loss==c("LASSO","RIDGE"))){
+        denominator <- apply(matrixXreg, 2, sd);
+        # No variability, substitute by 1
+        denominator[is.infinite(denominator)] <- 1;
+    }
+    else{
+        denominator <- NULL;
+    }
+
     #### Estimate parameters of the model ####
     if(is.null(parameters)){
         #### Add AR and I elements in the regression ####
@@ -1627,16 +1619,86 @@ alm <- function(formula, data, subset, na.action,
             print_level[] <- 0;
         }
 
+        # LASSO / RIDGE estimation
+        # Modify response to get correct initial estimates of parameters
         if(any(loss==c("LASSO","RIDGE"))){
-            denominator <- apply(matrixXreg, 2, sd);
-            # No variability, substitute by 1
-            denominator[is.infinite(denominator)] <- 1;
-            # # If it is lower than 1, then we are probably dealing with (0, 1). No need to normalise
-            # denominator[abs(denominator)<1] <- 1;
-            yDenominator <- max(sd(diff(y)),1);
+            y[] <- switch(distribution,
+                          "dlnorm" =,
+                          "dllaplace" =,
+                          "dls" =,
+                          "dlgnorm"=log(y),
+                          "dlogitnorm"=log(y/(1-y)),
+                          "dbcnorm" = bcTransform(y,lambdaBC),
+                          y);
         }
-        else{
-            denominator <- NULL;
+        # Preparations for LASSO/RIDGE
+        if(loss=="LASSO"){
+            if(interceptIsNeeded){
+                # Prepare matrices for the CF
+                matrixXregMeans <- colMeans(matrixXreg[otU, -1, drop=FALSE]);
+                matrixXreg <- (matrixXreg[, -1, drop=FALSE] -
+                                   matrix(matrixXregMeans,
+                                          obsInsample, nVariables-1, byrow=TRUE)) %*% diag(1/denominator[-1]);
+                yMean <- meanFast(y[otU]);
+                # Centre the response variable
+                y[] <- y - yMean;
+            }
+            else{
+                matrixXreg <- matrixXreg %*% diag(1/denominator);
+            }
+            # Change the initial estimate to RIDGE... It is closer to optimal than OLS
+            if(lambda!=1){
+                B <- solve(t(matrixXreg[otU,,drop=FALSE]) %*%
+                                 (matrixXreg[otU,,drop=FALSE]) +
+                                 lambda/(1-lambda) * diag(length(B)-1)) %*%
+                    t(matrixXreg[otU,,drop=FALSE]) %*% y[otU];
+            }
+            else{
+                B <- B[-1];
+                B[] <- 0;
+            }
+            # Upper bound is thousand times of RIDGE estimates of parameters
+            BUpper <- abs(B)*1000 + 1e-5;
+            BLower <- -BUpper - 1e-5;
+        }
+        else if(loss=="RIDGE"){
+            if(interceptIsNeeded){
+                # Prepare matrices for RIDGE
+                matrixXregMeans <- colMeans(matrixXreg[otU, -1, drop=FALSE]);
+                matrixXregModified <- (matrixXreg[otU, -1, drop=FALSE] -
+                                           matrix(matrixXregMeans,
+                                                  obsInsample, nVariables-1, byrow=TRUE)) %*% diag(1/denominator[-1]);
+                yMean <- meanFast(y[otU]);
+                # RIDGE has closed form for its parameters. No need to do estimation
+                if(lambda!=1){
+                    B[-1] <- solve(t(matrixXregModified) %*%
+                                       (matrixXregModified) +
+                                       lambda/(1-lambda) * diag(length(B)-1)) %*%
+                        t(matrixXregModified) %*% (y[otU] - yMean);
+                    B[-1] <- B[-1]/denominator[-1];
+                }
+                else{
+                    # If lambda was equal to 1, we shrink everything to zero, RSS is ignored
+                    B[-1] <- 0;
+                }
+                B[1] <- yMean - sum(matrixXregMeans %*% B[-1]);
+            }
+            else{
+                matrixXregModified <- matrixXreg[otU, , drop=FALSE] %*% diag(1/denominator);
+                # RIDGE has closed form for its parameters. No need to do estimation
+                if(lambda!=1){
+                    B[] <- solve(t(matrixXregModified) %*%
+                                       (matrixXregModified) +
+                                       lambda/(1-lambda) * diag(length(B))) %*%
+                        t(matrixXregModified) %*% y[otU];
+                    B[] <- B[]/denominator[];
+                }
+                else{
+                    # If lambda was equal to 1, we shrink everything to zero, RSS is ignored
+                    B[] <- 0;
+                }
+            }
+            maxeval <- 1;
         }
 
         #### Define what to do with the maxeval ####
@@ -1653,10 +1715,11 @@ alm <- function(formula, data, subset, na.action,
             else{
                 maxeval <- length(B) * 40;
             }
-            # LASSO / RIDGE need more iterations to converge
             if(any(loss==c("LASSO","RIDGE"))){
-                # maxeval <- 1000;
-                maxeval <- length(B) * 80;
+                maxeval <- length(B) * 200;
+                if(lambda==1 || loss=="RIDGE"){
+                    maxeval <- 1;
+                }
             }
         }
         else{
@@ -1670,7 +1733,16 @@ alm <- function(formula, data, subset, na.action,
                       lb=BLower, ub=BUpper,
                       distribution=distribution, loss=loss, y=y, matrixXreg=matrixXreg,
                       recursiveModel=recursiveModel, denominator=denominator);
+        # res <- ccd(B, BUpper, BLower, CF, opts=list(algorithm=algorithm, xtol_rel=xtol_rel,
+        #                                             maxeval=maxeval, print_level=print_level,
+        #                                             maxtime=maxtime, xtol_abs=xtol_abs,
+        #                                             ftol_rel=ftol_rel, ftol_abs=ftol_abs),
+        #            distribution=distribution, loss=loss, y=y, matrixXreg=matrixXreg,
+        #            recursiveModel=recursiveModel, denominator=denominator);
         if(recursiveModel){
+            if(print_level_hidden>0){
+                print(res);
+            }
             res2 <- nloptr(res$solution, CF,
                            opts=list(algorithm=algorithm, xtol_rel=xtol_rel, maxeval=maxeval, print_level=print_level,
                                 maxtime=maxtime, xtol_abs=xtol_abs, ftol_rel=ftol_rel, ftol_abs=ftol_abs),
@@ -1684,9 +1756,35 @@ alm <- function(formula, data, subset, na.action,
         B[] <- res$solution;
         CFValue <- res$objective;
 
-        # A hack for LASSO / RIDGE, lambda==1 and intercept
-        if(any(loss==c("LASSO","RIDGE")) && lambda==1 && interceptIsNeeded){
-            B[-1] <- 0;
+        # Fixes for LASSO, reverting to the original variables
+        if(loss=="LASSO"){
+            if(lambda==1){
+                B[] <- 0;
+            }
+            else{
+                B[] <- B / denominator[-1];
+            }
+            if(interceptIsNeeded){
+                y[] <- y + yMean;
+                B <- c(yMean - sum(matrixXregMeans %*% B), B);
+                matrixXreg <- cbind(1,(matrixXreg %*% diag(denominator[-1])) +
+                                        matrix(matrixXregMeans, obsInsample, nVariables-1, byrow=TRUE));
+            }
+            else{
+                matrixXreg <- matrixXreg %*% diag(denominator);
+            }
+        }
+
+        # Return the response variable to the original value
+        if(any(loss==c("LASSO","RIDGE"))){
+            y[] <- switch(distribution,
+                          "dlnorm" =,
+                          "dllaplace" =,
+                          "dls" =,
+                          "dlgnorm"=exp(y),
+                          "dlogitnorm"=exp(y)/(1-exp(y)),
+                          "dbcnorm" = bcTransformInv(y,lambdaBC),
+                          y);
         }
 
         if(print_level_hidden>0){
